package testGrid;

import java.awt.Color;
import java.util.Random;

import unit4.gridLib.Grid;

public class TestGrid3 {

	public static void main(String[] args) {
		int maxRow = 25;
    	int maxCol = 40;
    	GameOfLife game = new GameOfLife(maxRow, maxCol);
    	game.setCell(maxRow / 2, maxCol / 2, true);
    	game.setCell(maxRow / 2 - 1, maxCol / 2 + 1, true);
    	game.setCell(maxRow / 2 - 2, maxCol / 2 + 1, true);
    	game.setCell(maxRow / 2 - 2, maxCol / 2, true);
    	game.setCell(maxRow / 2 - 2, maxCol / 2 - 1, true);
    	Grid g = new Grid(maxRow, maxCol, 20, 20);
    	game.setCell(maxRow / 2, maxCol / 2, true);
    	g.setCellColor(maxRow / 2, maxCol / 2, Color.green);
    	g.show();
    	drawGrid(game, g);
    	
    	while(true) {
    		
    		game.calculateNextGeneration();
    		g.delay(500);
    		drawGrid(game, g);
    	}
	}
	public static void drawGrid(GameOfLife game, Grid g) {
		int maxRow = game.getMaxRows();
    	int maxCol = game.getMaxCols();
    	for(int row = 0; row < maxRow; row++)
            for(int col = 0; col < maxCol; col++) {
            	if (game.getCell(row, col))
            		g.setCellColor(row, col, Color.green);
            	else
            		g.setCellColor(row, col, new Color(0,0,0,0));
            }
	}
}
class GameOfLife{
    private boolean[][] board;
    private static Random rnd = new Random();
    public GameOfLife(int maxRows, int maxCols) {
    	this.board = new boolean[maxRows][maxCols];
    	
    }
    public void init() {
    	int maxRow = this.board.length;
    	int maxCol = this.board[0].length;
    	for(int row = 0; row < maxRow; row++)
            for(int col = 0; col < maxCol; col++)
            	this.board[row][col] = false;
    }
    public int getMaxRows() {
    	return this.board.length;
    }
    public int getMaxCols() {
    	return this.board[0].length;
    }
    public boolean getCell(int row, int col) {
    	return this.board[row][col];
    }
    public void setCell(int row, int col, boolean value) {
    	int maxRows = this.board.length;
    	int maxCols = this.board[0].length;
    	if (row >=0 && row < maxRows && col >= 0 && col < maxCols)
    		board[row][col] = value;
    }
    public void calculateNextGeneration() {
    	int maxRows = this.board.length;
    	int maxCols = this.board[0].length;
    	int neighbors;
    	boolean[][] nextBoard = new boolean[maxRows][maxCols];
    	for(int row = 0; row < maxRows; row++)
            for(int col = 0; col < maxCols; col++)
            	nextBoard[row][col] = this.board[row][col];
    	for(int row = 0; row < maxRows; row++)
            for(int col = 0; col < maxCols; col++) { 
            	neighbors = getNeighbors(nextBoard, row, col);
            	// For a space that is 'populated':
                // Each cell with one or no neighbors dies, as if by solitude.
            	// Each cell with four or more neighbors dies, as if by over population.
            	// Each cell with two or three neighbors survives.
            	if (nextBoard[row][col]) {
            		if (neighbors <= 1 || neighbors >= 4)
            			this.board[row][col] = false;
            	}
            	// For a space that is 'empty' or 'unpopulated':
            	// Each cell with three neighbors becomes populated.
            	else 
            		if (neighbors == 3){
            			this.board[row][col] = true;
            		}
            	
                
            }
    	
    }
    
      
    private static int getNeighbors(boolean[][] nextBoard, int rowPos, int colPos) {
    	int maxRows = nextBoard.length;
    	int maxCols = nextBoard[0].length;
    	int alive = 0;
    	for(int row = rowPos - 1; row <= rowPos + 1; row++)
			for(int col = colPos - 1; col <= colPos + 1; col++)
				if (row >=0 && row < maxRows && col >= 0 && col < maxCols)
					if(nextBoard[row][col])
						alive++;
    	if (nextBoard[rowPos][colPos])
    		return alive - 1;
    	return alive;
    }
	
}